/* ecserver.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "daemonProgram.h"
#include "database.h"
#include "format.h"

#include <unistd.h>
#include <iostream.h>

extern void encode();

class ECServerMain : public DaemonProgram {
public:
  ECServerMain();

  virtual void run_daemon();

protected:
  virtual void help();
};

ECServerMain::
ECServerMain() : DaemonProgram("ecserver") {
}

void ECServerMain::
run_daemon() {
  // Set our own priority as nice as possible.  We'll be running in
  // the background.
  if (nice(20) < 0) {
    perror("nice");
  }

  if (!music_db->has_sequence("encode_id_seq")) {
    music_db->create_sequence("encode_id_seq");
  }

  encode();
}

void ECServerMain::
help() {
  cerr <<
    "\n"
    "This is a daemon process that monitors the Paloma database for any newly\n"
    "loaded sound files and encodes them into MP3 form.\n";
  DaemonProgram::help();
}

int
main(int argc, char *argv[]) {
  ECServerMain m;
  m.command_line(argc, argv);
  m.init();
  m.run();
  return (0);
}
