/* mkdir_p.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"
#include "mkdir_p.h"

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <errno.h>

void
mkdir_p(const string &pathname) {
  if (mkdir(pathname.c_str(), 0777) < 0) {
    if (errno == ENOENT) {
      // If we couldn't make the whole directory because of ENOENT (a
      // parent directory doesn't exist), then try recursively to make
      // the immediate parent directory.

      int slash = pathname.rfind('/');
      if (slash != string::npos) {
	string prefix = pathname.substr(0, slash);
	mkdir_p(prefix);
	mkdir(pathname.c_str(), 0777);
      }
    }
  }
}
