/* form.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "form.h"
#include "formDef.h"
#include "formWidget.h"
#include "widgetSize.h"

Form::
Form() {
  _data = NULL;
  _def = NULL;
}

Form::
~Form() {
}

Form::
operator Gtk_Widget & () {
  // If the form has the "scrollbar" flag, the Form is the scrolled
  // window that contains the table; otherwise, it's the table itself.
  if (_def != NULL && _def->_scrollbar) {
    return _sc_window;
  } else {
    return _table;
  }
}

void Form::
layout_form(int form_id, Dataset &data) {
  // Don't try to lay a form out twice.
  assert(_data == NULL);
  _data = &data;
  _def = FormDef::get_form(form_id);
  assert(_def != NULL);
  _table.resize(_def->_num_rows, _def->_num_columns);
  _table.set_homogeneous(_def->_homogeneous);

  FormDef::Widgets::const_iterator wi;

  for (wi = _def->_widgets.begin(); wi != _def->_widgets.end(); ++wi) {
    FormWidget::Instance instance = (*wi).make_instance(_data);
    _table.attach(manage(instance._widget), 
		  (*wi)._place_left, (*wi)._place_right,
		  (*wi)._place_top, (*wi)._place_bottom,
    		  (*wi)._xoptions, (*wi)._yoptions,
		  (*wi)._xpadding, (*wi)._ypadding);
    instance._widget->show();

    _instances.push_back(instance);
  }

  if (_def->_scrollbar) {
    // Put the table inside a scrolling window.
    request_initial_size(_sc_window,
			 _def->_x_size_pixels, _def->_y_size_pixels);
    _sc_window.set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    _sc_window.add_with_viewport(_table);
    _table.show();

  } else {
    // If we're not in a scrolling window, at least give us some elbow
    // room.
    _table.set_border_width(8);
  }
}

void Form::
show() {
  if (_def != NULL) {
    if (_def->_scrollbar) {
      _sc_window.show();
    } else {
      _table.show();
    }
  }
}

void Form::
update() {
  Instances::iterator ii;
  for (ii = _instances.begin(); ii != _instances.end(); ++ii) {
    (*ii).update(this);
  }
}

void Form::
get_variables(VariableDef &vars) const {
  Instances::const_iterator ii;
  for (ii = _instances.begin(); ii != _instances.end(); ++ii) {
    string internal_name = (*ii)._def->get_internal_name();
    if (!internal_name.empty()) {
      DataCell val = (*ii).get_value();

      // $var expands to the quoted value of the field, suitable for
      // testing in a SQL command.

      // $var.u expands to the unquoted value of the field.

      vars[internal_name] = val.q_str();
      vars[internal_name + ".u"] = val.get_text();
    }
  }
}

double Form::
get_ratio_slider(TableID table_id, int field_id) const {
  TableFieldKey key(table_id, field_id);
  RatioSliderValues::const_iterator rsvi = _ratio_slider_values.find(key);

  if (rsvi != _ratio_slider_values.end()) {
    return (*rsvi).second;
  }
  return 0.0;
}

void Form::
set_ratio_slider(TableID table_id, int field_id, double value) {
  TableFieldKey key(table_id, field_id);
  _ratio_slider_values[key] = value;
}

const FormDef &Form::
get_def() const {
  return *_def;
}

